module.exports.TabsManager     = require("./TabsManager.js");

const idle  = (function(){
    return {  
        start(opts) {
            this.options           = {};
            this.options.timeout   = opts.timeout  || 40;
            this.options.inactive  = opts.inactive || 5;
            this.idle              = "active";
            localStorage.idleTime  = 0;
            this.listenChannel();
            this.startTimer();
            this.setEvents();      
        },

        listenChannel(){
            let channel = new BroadcastChannel(`dw-idle-handler`);
            channel.onmessage = ((e)=>{
                if(e.data == "destroyTimers"){
                    this.destroyTimer();
                }
            });
        },

        startTimer(){
            if(!this._timer){
                this._timer = setInterval(() =>{
                    localStorage.idleTime = this.currentTime() + 1;
                    this.onTimerChange();
                } , 60000);
            }
        },

        destroyTimer(){
            this.tab = TabsManager.getTab();
            if(this._timer != null && this.tab.lastFocus == true != this.tab.focus == false && TabsManager.getTabs().open.length > 1){
                this._timer = null
                clearInterval(this._timer);
            }
        },

        currentTime(){
            return parseInt(localStorage.idleTime);
        },

        onTimerChange(){
            if(this.currentTime() == this.options.timeout){
                this.notify("timeout");
            } 
            else if(this.currentTime() >= this.options.inactive && this.idle == "active"){
                this.notify("inactive");
            }
        },

        setEvents(){
            window.addEventListener('click',      () =>{ this.onWindowInteraction() });
            window.addEventListener('keypress',   () =>{ this.onWindowInteraction() });
            window.addEventListener('mouseover',  () =>{ this.onWindowInteraction() });
            window.addEventListener('focus',      () =>{ this.startTimer() });
            
            window.addEventListener('blur',       () =>{
                let channel = new BroadcastChannel(`dw-idle-handler`);
                channel.postMessage("destroyTimers");
            });
        },

        onWindowInteraction(){
            if(this.idle == "inactive"){
                this.notify("active");
            }
            localStorage.idleTime = 0;
        }, 

        notify(state){
            let State   = state.charAt(0).toUpperCase() + state.substring(1);
            let channel = new BroadcastChannel(`on${State}`);
            channel.postMessage({state : state});
            this.idle   = state;
        },
        
        on(name,callback){
            let Name    = name.charAt(0).toUpperCase() + name.substring(1);
            let channel = new BroadcastChannel(`on${Name}`);
            channel.onmessage = ((e) =>{
               callback(e);
            });
        },
    };
}());

module.exports = idle;
